@@ -53,33 +53,33 @@ static void open_dir					(GtkWidget *widget,
 							 GtkCTreeNode *node, 
 							 gint n,
                                                          gpointer data);
-static void entry_set_file				(GtkWidget *widget, 
+static gboolean entry_set_file				(GtkWidget *widget, 
 							 GdkEventKey *key, 
 							 gpointer data);
 static void real_set_file				(GtkWidget *widget, 
 							 gpointer data);
-static void set_filter					(GtkWidget *widget, 
+static gboolean set_filter				(GtkWidget *widget, 
                                                          GdkEventKey *key,
 							 gpointer data);
 static gboolean select_icon				(GtkIconList *iconlist, 
             						 GtkIconListItem *icon,
             						 GdkEvent *event, 
 							 gpointer data);
-static void insert_text     				(GtkEditable *editable,
-                 					 const gchar *new_text,
-                 					 gint  new_text_length,
-                 					 gint  *position,
-                 					 gpointer data);
+static gboolean insert_text                             (GtkEditable *editable, 
+                                                         const gchar *new_text,
+                                                         gint  new_text_length, 
+                                                         gint  *position,
+                                                         gpointer data);
 static void init_history_combo				(GtkIconFileSel *filesel, 
 							 const gchar *curr_dir);
 static void update_history_combo			(GtkIconFileSel *filesel, 
 							 const gchar *curr_dir);
 static void go_to_history				(GtkEntry *entry,
 							 gpointer data);
-static void combo_changed				(GtkWidget *widget, 
+static gboolean combo_changed				(GtkWidget *widget, 
 							 GtkWidget *child,
 							 gpointer data);
-static void entry_key_press				(GtkWidget *widget, 
+static gboolean entry_key_press				(GtkWidget *widget, 
 							 GdkEventKey *event, 
 							 gpointer data);
 static gchar *get_real_path				(const gchar *full_path);
@@ -145,25 +145,6 @@ gtk_icon_file_selection_class_init (GtkIconFileSelClass *klass)
   object_class->destroy = gtk_icon_file_selection_destroy;
 }
 
-GtkWidget *
-pixmap_button (GtkWidget *pixmap, const gchar *text)
-{
-  GtkWidget *button;
-  GtkWidget *bbox1, *bbox2;
-                                                                                
-  button = gtk_button_new ();
-  bbox1 = gtk_hbox_new(FALSE, 0);
-  gtk_container_add(GTK_CONTAINER(button), bbox1);
-                                                                                
-  bbox2 = gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(bbox1), bbox2, TRUE, FALSE, 2);
-  gtk_box_pack_start(GTK_BOX(bbox2), pixmap, FALSE, FALSE, 0);
-  gtk_box_pack_end(GTK_BOX(bbox2), gtk_label_new(text), FALSE, FALSE, 2);
-  gtk_widget_show_all(bbox1);
-                                                                                
-  return button;
-}
-
 static void
 up_clicked(GtkWidget *widget, gpointer data)
 {
@@ -323,13 +304,14 @@ gtk_icon_file_selection_init (GtkIconFileSel *filesel)
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);
 
+/* FIXME
   filesel->dir_tree = gtk_dir_tree_new();
   GTK_DIR_TREE(filesel->dir_tree)->show_hidden = TRUE;
   gtk_container_add(GTK_CONTAINER(scrolled_window), filesel->dir_tree);
   gtk_widget_show(filesel->dir_tree);
 
-
   gtk_box_pack_start(GTK_BOX(hbox), gtk_vseparator_new(), TRUE, TRUE, 0);
+*/
 
   filesel->list_window = scrolled_window=gtk_scrolled_window_new(NULL, NULL);
   gtk_box_pack_start(GTK_BOX(hbox), scrolled_window, TRUE, TRUE, 0);
@@ -389,8 +371,10 @@ gtk_icon_file_selection_init (GtkIconFileSel *filesel)
   gtk_signal_connect(GTK_OBJECT(filesel->filter_entry), "key_press_event",
                      GTK_SIGNAL_FUNC(set_filter), filesel);
 
+/* FIXME
   gtk_signal_connect(GTK_OBJECT(filesel->filter_entry), "insert_text",
                      GTK_SIGNAL_FUNC(insert_text), NULL);
+*/
 
   box = gtk_vbutton_box_new();
   gtk_table_attach_defaults(GTK_TABLE(table), box, 3, 4, 0, 2);
@@ -402,7 +386,7 @@ gtk_icon_file_selection_init (GtkIconFileSel *filesel)
   gdk_pixmap_unref(pixmap);
   gdk_bitmap_unref(mask);
 
-  filesel->ok_button = pixmap_button(wpixmap, "OK");
+  filesel->ok_button = gtk_button_new_from_stock(GTK_STOCK_OK);
   gtk_box_pack_end (GTK_BOX (box), filesel->ok_button, TRUE, TRUE, 0);
   gtk_widget_show(filesel->ok_button);
 
@@ -415,7 +399,7 @@ gtk_icon_file_selection_init (GtkIconFileSel *filesel)
   gdk_pixmap_unref(pixmap);
   gdk_bitmap_unref(mask);
 
-  filesel->cancel_button = pixmap_button(wpixmap, "Cancel");
+  filesel->cancel_button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
   gtk_box_pack_end (GTK_BOX (box), filesel->cancel_button, TRUE, TRUE, 0);
   gtk_widget_show(filesel->cancel_button);
 
@@ -438,6 +422,7 @@ gtk_icon_file_selection_destroy(GtkObject *object)
 void
 gtk_icon_file_selection_show_tree(GtkIconFileSel *filesel, gboolean show)
 {
+/* FIXME
   if(show == filesel->show_tree) return;
 
   filesel->show_tree = show;
@@ -461,25 +446,28 @@ gtk_icon_file_selection_show_tree(GtkIconFileSel *filesel, gboolean show)
     gtk_widget_hide(filesel->tree_window);
     gtk_widget_set_usize(filesel->list_window, 550, 250);
   } 
+*/
 }
 
-static void
+static gboolean
 insert_text     (GtkEditable *editable,
                  const gchar *new_text,
                  gint         new_text_length,
                  gint        *position,
                  gpointer data)
 {
+  g_object_ref(editable);
   gtk_signal_emit_stop_by_name(GTK_OBJECT(editable), "insert_text");
   if(new_text[0] != ' '){
-     GTK_EDITABLE_CLASS (gtk_type_class(GTK_TYPE_ENTRY))->insert_text(editable,
+     GTK_EDITABLE_CLASS (gtk_type_class(GTK_TYPE_EDITABLE))->insert_text(editable,
                                                               new_text,
-                                                              new_text_length, 
+                                                              new_text_length,
                                                               position);
-
-
+                                                                               
+                                                                               
   }
-
+  g_object_unref(editable);
+  return TRUE;
 }
 
 static gboolean 
@@ -532,9 +520,11 @@ select_icon(GtkIconList *iconlist,
 
   if((mods & GDK_BUTTON1_MASK) && event->type == GDK_2BUTTON_PRESS){
     gtk_label_set_text(GTK_LABEL(filesel->path_label), "Scanning...");
-    if(filesel->show_tree)
+    if(filesel->show_tree){
+/* FIXME
       return_val = gtk_dir_tree_open_dir(GTK_DIR_TREE(filesel->dir_tree), real_path);
-    else
+*/
+    } else
       return_val = gtk_file_list_open_dir(GTK_FILE_LIST(filesel->file_list), real_path);
 
     update_history_combo(filesel, real_path);
@@ -547,16 +537,18 @@ select_icon(GtkIconList *iconlist,
   return (!return_val);
 }
 
-static void
+static gboolean 
 entry_set_file(GtkWidget *widget, GdkEventKey *key, gpointer data)
 {
   GtkIconFileSel *filesel = GTK_ICON_FILESEL(data);
 
-  if(key->keyval != GDK_Return && key->keyval != GDK_KP_Enter) return;
+  if(key->keyval != GDK_Return && key->keyval != GDK_KP_Enter) return FALSE;
 
 /*  real_set_file(widget, data);
 */
   gtk_signal_emit_by_name(GTK_OBJECT(filesel->ok_button), "clicked", filesel);
+
+  return FALSE;
 }
 
 static void
@@ -565,7 +557,8 @@ real_set_file(GtkWidget *widget, gpointer data)
   GtkIconFileSel *filesel;
   GtkIconListItem *item;
   GList *list;
-  gchar *c, *last, *text;
+  const gchar *c;
+  gchar *last, *text;
   gchar *folder;
   gchar *file;
   gint nlen, file_len;
@@ -618,16 +611,18 @@ real_set_file(GtkWidget *widget, gpointer data)
   g_free(last);
 }
 
-static void
+static gboolean 
 set_filter(GtkWidget *widget, GdkEventKey *key, gpointer data)
 {
   GtkIconFileSel *filesel;
 
-  if(key->keyval != GDK_Return && key->keyval != GDK_KP_Enter) return;
+  if(key->keyval != GDK_Return && key->keyval != GDK_KP_Enter) return FALSE;
 
   filesel = (GtkIconFileSel *)data;
   gtk_file_list_set_filter(GTK_FILE_LIST(filesel->file_list), 
                            gtk_entry_get_text(GTK_ENTRY(widget)));
+
+  return TRUE;
 }
 
 static void
@@ -672,7 +667,7 @@ gint
 gtk_icon_file_selection_open_dir(GtkIconFileSel *filesel, const gchar *path)
 {
   DIR *dir;
-  gint return_val;
+  gint return_val = TRUE;
   gchar *real_path = NULL;
  
   if(!path) return FALSE;
@@ -686,9 +681,11 @@ gtk_icon_file_selection_open_dir(GtkIconFileSel *filesel, const gchar *path)
 
   gtk_label_set_text(GTK_LABEL(filesel->path_label), "Scanning...");
 
-  if(filesel->show_tree)
+  if(filesel->show_tree){
+/* FIXME
     return_val = gtk_dir_tree_open_dir(GTK_DIR_TREE(filesel->dir_tree), real_path);
-  else
+*/
+  } else
     return_val = gtk_file_list_open_dir(GTK_FILE_LIST(filesel->file_list), real_path);
 
   gtk_label_set_text(GTK_LABEL(filesel->path_label), real_path);
@@ -703,7 +700,9 @@ gtk_icon_file_selection_open_dir(GtkIconFileSel *filesel, const gchar *path)
 void
 gtk_icon_file_selection_show_hidden(GtkIconFileSel *filesel, gboolean visible)
 {
+/* FIXME
     GTK_DIR_TREE(filesel->dir_tree)->show_hidden = visible;
+*/
     GTK_FILE_LIST(filesel->file_list)->show_hidden = visible;
 }
 
@@ -858,7 +857,7 @@ update_history_combo(GtkIconFileSel *filesel, const gchar *current_directory)
 static void
 go_to_history(GtkEntry *entry, gpointer data)
 {
-  gchar *path;
+  const gchar *path;
   gchar *real_path;
 
   path = gtk_entry_get_text(entry);
@@ -870,7 +869,7 @@ go_to_history(GtkEntry *entry, gpointer data)
   g_free(real_path);
 }
 
-static void
+static gboolean 
 combo_changed(GtkWidget *widget, GtkWidget *child, gpointer data)
 {
   GtkCombo *combo;
@@ -884,10 +883,12 @@ combo_changed(GtkWidget *widget, GtkWidget *child, gpointer data)
   gtk_signal_handler_block (GTK_OBJECT (combo->list), combo->list_change_id);
   go_to_history(entry, filesel);
   gtk_signal_handler_unblock (GTK_OBJECT (combo->list), combo->list_change_id);
+
+  return TRUE;
 }
 
 
-static void
+static gboolean 
 entry_key_press(GtkWidget *widget, 
 	        GdkEventKey *event, 
 		gpointer data)
@@ -897,7 +898,9 @@ entry_key_press(GtkWidget *widget,
   if(event->keyval == GDK_Return){
      gtk_signal_emit_stop_by_name(GTK_OBJECT(entry), "key_press_event");
      go_to_history(entry, data);
+     return TRUE;
   }
+  return FALSE;
 } 
 
 const gchar *
