@@ -1,5 +1,5 @@
 /* gtkiconlist - gtkiconlist widget for gtk+
- * Copyright 1999-DEFAULT_COL_SPACING01  Adrian E. Feiguin <feiguin@ifir.edu.ar>
+ * Copyright 1999-2001  Adrian E. Feiguin <feiguin@ifir.edu.ar>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,8 +22,10 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include <pango/pango.h>
 #include "gtkitementry.h"
 #include "gtkiconlist.h"
+#include "gtkextra-marshal.h"
 #include <math.h>
 
 #define DEFAULT_ROW_SPACING 	4
@@ -40,6 +42,10 @@
                        GDK_BUTTON_RELEASE_MASK)
 
 /* Signals */
+
+extern void 
+_gtkextra_signal_emit(GtkObject *object, guint signal_id, ...);
+
 enum{
       SELECT_ICON,                       
       UNSELECT_ICON,                       
@@ -50,33 +56,16 @@ enum{
       LAST_SIGNAL,
 };
 
-static void
-gtk_iconlist_marshal_BOOL__POINTER_POINTER (GtkObject *object,
-                                            GtkSignalFunc func,
-                                            gpointer func_data,
-                                            GtkArg * args);
-
-typedef gboolean (*GtkIconListSignal1) (GtkObject *object,
-                                    	GtkIconListItem *arg1, gchar *arg2,
-                                    	gpointer user_data);
-
-
 static guint signals[LAST_SIGNAL] = {0};
 
 static void gtk_icon_list_class_init 		(GtkIconListClass *class);
 static void gtk_icon_list_init 			(GtkIconList *icon_list);
-static void gtk_icon_list_finalize 		(GtkObject *object);
 static void gtk_icon_list_destroy 		(GtkObject *object);
 
 static void gtk_icon_list_size_allocate		(GtkWidget *widget,
                                                  GtkAllocation *allocation);
 
 static void gtk_icon_list_realize		(GtkWidget *widget);
-static void gtk_icon_list_map			(GtkWidget *widget); 
-static void gtk_icon_list_draw 			(GtkWidget *widget, 
-						 GdkRectangle *area);
-static void gtk_icon_list_paint 		(GtkWidget *widget, 
-						 GdkRectangle *area);
 static gint gtk_icon_list_expose		(GtkWidget *widget, 
 						 GdkEventExpose *event);
 static gint gtk_icon_list_button_press		(GtkWidget *widget, 
@@ -121,10 +110,27 @@ static gint icon_key_press			(GtkWidget *widget,
 						 GdkEventKey *key, 
 						 gpointer data);
 static gint sort_list				(gpointer a, gpointer b);
-static void pixmap_destroy			(GtkPixmap* pixmap);
+static void pixmap_destroy                      (GtkPixmap* pixmap);
+
 
 static GtkFixedClass *parent_class = NULL;
 
+static inline guint STRING_WIDTH(GtkWidget *widget,
+                                 PangoFontDescription *font, const gchar *text)
+{
+  PangoRectangle rect;
+  PangoLayout *layout;
+
+  layout = gtk_widget_create_pango_layout (widget, text);
+  pango_layout_set_font_description (layout, font);
+
+  pango_layout_get_pixel_extents (layout, NULL, &rect);
+
+  g_object_unref(G_OBJECT(layout));
+  return rect.width;
+}
+
+
 GtkType
 gtk_icon_list_get_type (void)
 {
@@ -132,7 +138,7 @@ gtk_icon_list_get_type (void)
 
   if (!icon_list_type)
     {
-      GtkTypeInfo icon_list_info =
+      static const GtkTypeInfo icon_list_info =
       {
 	"GtkIconList",
 	sizeof (GtkIconList),
@@ -149,31 +155,41 @@ gtk_icon_list_get_type (void)
   return icon_list_type;
 }
 
-GtkType
-gtk_icon_list_item_get_type (void)
+static GtkIconListItem*
+gtk_icon_list_item_copy (const GtkIconListItem *item)
 {
-  static GtkType icon_list_item_type = 0;
+  GtkIconListItem *new_item;
 
-  if (!icon_list_item_type)
-    {
-      GtkTypeInfo icon_list_item_info =
-      {
-       "GtkIconListItem",
-       0,
-       0,
-       (GtkClassInitFunc) NULL,
-       (GtkObjectInitFunc) NULL,
-       /* reserved 1*/ NULL,
-        /* reserved 2 */ NULL,
-        (GtkClassInitFunc) NULL,
-      };
+  g_return_val_if_fail (item != NULL, NULL);
 
-      icon_list_item_type = gtk_type_unique (GTK_TYPE_BOXED,
-                                             &icon_list_item_info);
-    }
-  return icon_list_item_type;
+  new_item = g_new (GtkIconListItem, 1);
+
+  *new_item = *item;
+
+  return new_item;
 }
 
+static void
+gtk_icon_list_item_free (GtkIconListItem *item)
+{
+  g_return_if_fail (item != NULL);
+
+  g_free (item);
+}
+
+
+GType
+gtk_icon_list_item_get_type (void)
+{
+  static GType icon_list_item_type;
+
+  if(!icon_list_item_type)
+  {
+    icon_list_item_type = g_boxed_type_register_static("GtkIconListItem", (GBoxedCopyFunc)gtk_icon_list_item_copy, (GBoxedFreeFunc)gtk_icon_list_item_free);
+  }
+  return icon_list_item_type;
+}
+ 
 
 static void
 gtk_icon_list_class_init (GtkIconListClass *klass)
@@ -181,102 +197,76 @@ gtk_icon_list_class_init (GtkIconListClass *klass)
   GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
 
-  parent_class = gtk_type_class (gtk_fixed_get_type ());
+  parent_class = gtk_type_class (GTK_TYPE_FIXED);
 
   object_class = (GtkObjectClass *) klass;
   widget_class = (GtkWidgetClass *) klass;
 
-  object_class->finalize = gtk_icon_list_finalize;
   object_class->destroy = gtk_icon_list_destroy;
 
   widget_class->realize = gtk_icon_list_realize;
 
   widget_class->size_allocate = gtk_icon_list_size_allocate;
 
-  widget_class->draw = gtk_icon_list_draw;
-  widget_class->map = gtk_icon_list_map;
   widget_class->expose_event = gtk_icon_list_expose;
   widget_class->button_press_event = gtk_icon_list_button_press;
 
   signals[SELECT_ICON] =
       gtk_signal_new("select_icon",
 		     GTK_RUN_LAST,
-		     object_class->type,
+		     GTK_CLASS_TYPE(object_class),
 		     GTK_SIGNAL_OFFSET(GtkIconListClass, select_icon),
-                     gtk_iconlist_marshal_BOOL__POINTER_POINTER,
+                     gtkextra_BOOLEAN__BOXED_BOXED,
 		     GTK_TYPE_BOOL, 2,
-		     GTK_TYPE_ICON_LIST_ITEM, GTK_TYPE_GDK_EVENT); 
+		     GTK_TYPE_ICON_LIST_ITEM, GDK_TYPE_EVENT); 
 
   signals[UNSELECT_ICON] =
       gtk_signal_new("unselect_icon",
 		     GTK_RUN_FIRST,
-		     object_class->type,
+		     GTK_CLASS_TYPE(object_class),
 		     GTK_SIGNAL_OFFSET(GtkIconListClass, unselect_icon),
-                     gtk_marshal_NONE__POINTER_POINTER,
+                     gtkextra_VOID__BOXED_BOXED,
 		     GTK_TYPE_NONE, 2,
-		     GTK_TYPE_ICON_LIST_ITEM, GTK_TYPE_GDK_EVENT); 
+		     GTK_TYPE_ICON_LIST_ITEM, GDK_TYPE_EVENT); 
 
   signals[TEXT_CHANGED] =
       gtk_signal_new("text_changed",
 		     GTK_RUN_LAST,
-		     object_class->type,
+		     GTK_CLASS_TYPE(object_class),
 		     GTK_SIGNAL_OFFSET(GtkIconListClass, text_changed),
-                     gtk_iconlist_marshal_BOOL__POINTER_POINTER,
+                     gtkextra_BOOLEAN__BOXED_STRING,
 		     GTK_TYPE_BOOL, 2,
 		     GTK_TYPE_ICON_LIST_ITEM, GTK_TYPE_STRING); 
 
   signals[ACTIVATE_ICON] =
       gtk_signal_new("activate_icon",
 		     GTK_RUN_LAST,
-		     object_class->type,
+		     GTK_CLASS_TYPE(object_class),
 		     GTK_SIGNAL_OFFSET(GtkIconListClass, activate_icon),
-                     gtk_marshal_BOOL__POINTER,
+                     gtkextra_BOOLEAN__BOXED,
 		     GTK_TYPE_BOOL, 1,
 		     GTK_TYPE_ICON_LIST_ITEM); 
 
   signals[DEACTIVATE_ICON] =
       gtk_signal_new("deactivate_icon",
 		     GTK_RUN_LAST,
-		     object_class->type,
+		     GTK_CLASS_TYPE(object_class),
 		     GTK_SIGNAL_OFFSET(GtkIconListClass, deactivate_icon),
-                     gtk_marshal_BOOL__POINTER,
+                     gtkextra_BOOLEAN__BOXED,
 		     GTK_TYPE_BOOL, 1,
 		     GTK_TYPE_ICON_LIST_ITEM); 
 
   signals[CLICK_EVENT] =
       gtk_signal_new("click_event",
 		     GTK_RUN_LAST,
-		     object_class->type,
+		     GTK_CLASS_TYPE(object_class),
 		     GTK_SIGNAL_OFFSET(GtkIconListClass, click_event),
-                     gtk_marshal_NONE__POINTER,
+                     gtkextra_VOID__BOXED,
 		     GTK_TYPE_NONE, 1,
-		     GTK_TYPE_GDK_EVENT); 
-
-  gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
-
+		     GDK_TYPE_EVENT); 
  
 }
 
-static void
-gtk_iconlist_marshal_BOOL__POINTER_POINTER (GtkObject *object,
-                                            GtkSignalFunc func,
-                                            gpointer func_data,
-                                            GtkArg * args)
-{
-  GtkIconListSignal1 rfunc;
-  gboolean *veto;
-
-  veto = GTK_RETLOC_BOOL (args[2]);
-
-  rfunc = (GtkIconListSignal1) func;
-
-  *veto = (*rfunc) (object, 
-                    GTK_VALUE_POINTER (args[0]),
-                    GTK_VALUE_POINTER (args[1]),
-                    func_data);
-
-}
-
 void
 gtk_icon_list_freeze(GtkIconList *iconlist)
 {
@@ -314,7 +304,6 @@ reorder_icons(GtkIconList *iconlist)
   widget = GTK_WIDGET(iconlist);
 
   if(iconlist->freeze_count > 0) return;
-
 /*
   gdk_threads_enter();
 */
@@ -339,7 +328,7 @@ reorder_icons(GtkIconList *iconlist)
     switch(iconlist->mode){
       case GTK_ICON_LIST_TEXT_RIGHT:
         y += vspace;
-        if(y + vspace >= old_height-DEFAULT_COL_SPACING){
+        if(y + vspace >= old_height - DEFAULT_COL_SPACING){
                 x += hspace;
                 y = iconlist->row_spacing;
         }
@@ -348,7 +337,7 @@ reorder_icons(GtkIconList *iconlist)
       case GTK_ICON_LIST_ICON:
       default:
         x += hspace;
-        if(x + hspace >= old_width-DEFAULT_COL_SPACING){
+        if(x + hspace >= old_width - DEFAULT_COL_SPACING){
                 x = iconlist->col_spacing;
                 y += vspace;
         }
@@ -358,7 +347,6 @@ reorder_icons(GtkIconList *iconlist)
     icons = icons->next;
 
    }
-
 /*
   gdk_threads_leave();
 */
@@ -372,8 +360,8 @@ gtk_icon_list_move(GtkIconList *iconlist, GtkIconListItem *icon,
   GtkRequisition req1, req2;
   GtkRequisition req;
   GtkAllocation a;
-  gchar *text;
-  gint width, height;
+  const gchar *text;
+  gint old_width, old_height, width, height;
   gint old_x, old_y;
   gint size;
 
@@ -397,6 +385,9 @@ gtk_icon_list_move(GtkIconList *iconlist, GtkIconListItem *icon,
   if(iconlist->mode == GTK_ICON_LIST_ICON) 
           req2.width = req2.height = 0;
 
+  old_width = width = GTK_WIDGET(iconlist)->allocation.width;
+  old_height = height = GTK_WIDGET(iconlist)->allocation.height;
+
   gtk_fixed_move(GTK_FIXED(iconlist), icon->pixmap, 
                  x + req1.width/2 - icon->pixmap->requisition.width/2, 
                  y + iconlist->icon_border);
@@ -407,20 +398,18 @@ gtk_icon_list_move(GtkIconList *iconlist, GtkIconListItem *icon,
   icon->entry->allocation.y += (y - old_y);
   icon->entry->allocation.width = req2.width;
 
-  width = GTK_WIDGET(iconlist)->allocation.width;
-  height = GTK_WIDGET(iconlist)->allocation.height;
 
   switch(iconlist->mode){
    case GTK_ICON_LIST_TEXT_BELOW:
         text = gtk_entry_get_text(GTK_ENTRY(icon->entry));
-        size = gdk_string_width(icon->entry->style->font, text);
+        size = STRING_WIDTH(icon->entry, icon->entry->style->font_desc, text);
         size = MIN(size, req2.width);
 
   	gtk_fixed_move(GTK_FIXED(iconlist), icon->entry, 
         	        x - req2.width/2 + req1.width/2, 
                         y + req1.height + iconlist->icon_border);
 
-        if(y + req.height > height-DEFAULT_COL_SPACING) 
+        if(y + req.height > height) 
            height += req.height;
         break;
    case GTK_ICON_LIST_TEXT_RIGHT:
@@ -428,7 +417,7 @@ gtk_icon_list_move(GtkIconList *iconlist, GtkIconListItem *icon,
         	        x + req1.width + iconlist->icon_border, 
                         y + req1.height/2 - req2.height/2); 
 
-        if(x + req.width > width-DEFAULT_COL_SPACING) 
+        if(x + req.width > width) 
             width += req.width;
 	break;
    case GTK_ICON_LIST_ICON:
@@ -468,24 +457,26 @@ gtk_icon_list_realize(GtkWidget *widget)
 
   iconlist = GTK_ICON_LIST(widget);
 
-  gtk_widget_ensure_style(widget);
-
   style = gtk_style_copy(widget->style);
+
   style->bg[0] = iconlist->background;
   gtk_widget_set_style(widget, style);
-  gtk_style_set_background(widget->style, widget->window, GTK_STATE_NORMAL);
-  gtk_style_unref(style);
+  gtk_style_set_background(style, widget->window, GTK_STATE_NORMAL);
+  gtk_style_set_background(style, widget->window, GTK_STATE_ACTIVE);
 
   icons = iconlist->icons;
   while(icons){
     item = (GtkIconListItem *) icons->data;
     gtk_widget_draw(item->pixmap, NULL);
     if(iconlist->mode != GTK_ICON_LIST_ICON){
+      GtkStyle *style;
+
       gtk_widget_realize(item->entry);
-      gdk_gc_set_foreground(GTK_ITEM_ENTRY(item->entry)->bg_gc, 
-                            &iconlist->background);
-      gdk_gc_set_foreground(GTK_ITEM_ENTRY(item->entry)->fg_gc, 
-                            &widget->style->fg[GTK_STATE_NORMAL]);
+
+      style = gtk_style_copy(item->entry->style);
+      style->bg[GTK_STATE_ACTIVE] = iconlist->background;
+      style->bg[GTK_STATE_NORMAL] = iconlist->background;
+      gtk_widget_set_style(item->entry, style);
       gtk_widget_show(item->entry);
     }
     if(item->entry) gtk_widget_draw(item->entry, NULL);
@@ -506,6 +497,8 @@ gtk_icon_list_init (GtkIconList *icon_list)
   gdk_color_black(gtk_widget_get_colormap(widget), &widget->style->black);
   gdk_color_white(gtk_widget_get_colormap(widget), &widget->style->white);
 
+  gtk_fixed_set_has_window(GTK_FIXED(widget), TRUE);
+
   gtk_widget_set_events (widget, gtk_widget_get_events(widget)|
                          EVENTS_MASK);
 
@@ -536,70 +529,33 @@ sort_list(gpointer a, gpointer b)
   return (strcmp(itema->label, itemb->label));
 }
 
-static void
-gtk_icon_list_paint (GtkWidget *widget, GdkRectangle *area)
+static gboolean
+gtk_icon_list_expose (GtkWidget *widget, GdkEventExpose *event)
 {
   GtkIconList *icon_list;
 
   icon_list = GTK_ICON_LIST(widget);
 
-  if(!GTK_WIDGET_DRAWABLE(widget)) return;
+  if(!GTK_WIDGET_DRAWABLE(widget)) return FALSE;
 
-  GTK_WIDGET_CLASS(parent_class)->draw(widget, area);
+  gtk_paint_flat_box (widget->style, widget->window, GTK_STATE_NORMAL,
+                      GTK_SHADOW_NONE, &event->area, widget, "base", 0, 0, -1, -1);
+
+  GTK_WIDGET_CLASS(parent_class)->expose_event(widget, event);
 
   if(icon_list->active_icon && icon_list->active_icon->entry){
 	gdk_draw_rectangle(widget->window,
-                           widget->style->fg_gc[GTK_STATE_NORMAL],
+                           widget->style->black_gc,
                            FALSE,
-                           icon_list->active_icon->entry->allocation.x+2,
-                           icon_list->active_icon->entry->allocation.y+2,
-                           icon_list->active_icon->entry->allocation.width-4,
-                           icon_list->active_icon->entry->allocation.height-4);
+                           icon_list->active_icon->entry->allocation.x-2,
+                           icon_list->active_icon->entry->allocation.y-2,
+                           icon_list->active_icon->entry->allocation.width+4,
+                           icon_list->active_icon->entry->allocation.height+4);
   }
 
-}
-
-static void
-gtk_icon_list_draw (GtkWidget *widget, GdkRectangle *darea)
-{
-  GdkRectangle area;
-  if(darea)
-     area = *darea;
-  else
-   {
-     area.x = 0;
-     area.y = 0;
-     area.width = widget->allocation.width;
-     area.height = widget->allocation.height;
-   }
-
-/*  GTK_WIDGET_CLASS(parent_class)->draw (widget, &area);
-*/
-  gtk_icon_list_paint(widget, &area);
-
-}
-
-
-static gint
-gtk_icon_list_expose(GtkWidget *widget, GdkEventExpose *event)
-{
-  gtk_icon_list_paint(widget, &event->area);
   return FALSE;
 }
 
-static void
-gtk_icon_list_map (GtkWidget *widget)
-{
-  if(!GTK_WIDGET_MAPPED(widget)){
-    GTK_WIDGET_CLASS(parent_class)->map (widget);
-    GTK_WIDGET_SET_FLAGS(widget, GTK_MAPPED);
-
-/*
-    set_adjustments(widget, NULL);
-*/
-  }
-}
-
 static gint
 gtk_icon_list_button_press(GtkWidget *widget, GdkEventButton *event)
 {
@@ -612,6 +568,7 @@ gtk_icon_list_button_press(GtkWidget *widget, GdkEventButton *event)
 
   iconlist = GTK_ICON_LIST(widget);
 
+
   gtk_widget_get_pointer(widget, &x, &y);
   item = gtk_icon_list_get_icon_at(iconlist, x , y );
 
@@ -635,8 +592,9 @@ gtk_icon_list_button_press(GtkWidget *widget, GdkEventButton *event)
      case GTK_SELECTION_BROWSE:
         unselect_all(iconlist);
      case GTK_SELECTION_MULTIPLE:
-     case GTK_SELECTION_EXTENDED:
         select_icon(iconlist, item, (GdkEvent *)event);
+     case GTK_SELECTION_NONE:
+        break;
    }
 
   return FALSE;
@@ -648,22 +606,35 @@ deactivate_entry(GtkIconList *iconlist)
 {
   GdkGC *gc;
   GtkEntry *entry;
-  gboolean veto;
+  gboolean veto = TRUE;
 
   if(iconlist->active_icon) {
-     gtk_signal_emit(GTK_OBJECT(iconlist), signals[DEACTIVATE_ICON], 
+     _gtkextra_signal_emit(GTK_OBJECT(iconlist), signals[DEACTIVATE_ICON], 
                      iconlist->active_icon, &veto);
      if(!veto) return FALSE;
 
      entry = GTK_ENTRY(iconlist->active_icon->entry);
 
-     if(!entry || !GTK_WIDGET_REALIZED(entry)) return FALSE;
-
+     if(!entry || !GTK_WIDGET_REALIZED(entry)) return TRUE;
      gtk_entry_set_editable(entry, FALSE);    
-     gtk_entry_set_text(entry, iconlist->active_icon->entry_label);
      gtk_entry_select_region(entry, 0, 0);
-     gtk_entry_set_position(entry, 0);
+     gtk_item_entry_set_cursor_visible(GTK_ITEM_ENTRY(entry), FALSE);
 
+     switch(iconlist->mode){
+       case GTK_ICON_LIST_TEXT_RIGHT:
+         gtk_item_entry_set_text(GTK_ITEM_ENTRY(entry), 
+                                 iconlist->active_icon->entry_label,
+                                 GTK_JUSTIFY_LEFT);
+         break;
+       case GTK_ICON_LIST_TEXT_BELOW:
+         gtk_item_entry_set_text(GTK_ITEM_ENTRY(entry), 
+                                 iconlist->active_icon->entry_label,
+                                 GTK_JUSTIFY_CENTER);
+         break;
+       case GTK_ICON_LIST_ICON:
+       default:
+         break;
+     }
 
      if(GTK_WIDGET_REALIZED(iconlist->active_icon->entry)){
        gc = gdk_gc_new(GTK_WIDGET(iconlist)->window);
@@ -671,10 +642,10 @@ deactivate_entry(GtkIconList *iconlist)
        gdk_draw_rectangle(GTK_WIDGET(iconlist)->window,
                           gc,
                           FALSE,
-                          GTK_WIDGET(entry)->allocation.x+2,
-                          GTK_WIDGET(entry)->allocation.y+2,
-                          GTK_WIDGET(entry)->allocation.width-4,
-                          GTK_WIDGET(entry)->allocation.height-4);
+                          GTK_WIDGET(entry)->allocation.x-2,
+                          GTK_WIDGET(entry)->allocation.y-2,
+                          GTK_WIDGET(entry)->allocation.width+4,
+                          GTK_WIDGET(entry)->allocation.height+4);
        gdk_gc_unref(gc);
      }
 
@@ -693,7 +664,7 @@ select_icon(GtkIconList *iconlist, GtkIconListItem *item, GdkEvent *event)
 
   if(item == NULL) return;
 
-  gtk_signal_emit(GTK_OBJECT(iconlist), signals[SELECT_ICON], item, 
+  _gtkextra_signal_emit(GTK_OBJECT(iconlist), signals[SELECT_ICON], item, 
                   event, &veto);                        
 
   if(!veto) return;
@@ -702,22 +673,36 @@ select_icon(GtkIconList *iconlist, GtkIconListItem *item, GdkEvent *event)
     if(!deactivate_entry(iconlist)) return;
 
     if(item->entry && GTK_WIDGET_REALIZED(item->entry)){
-      gdk_gc_set_foreground(GTK_ITEM_ENTRY(item->entry)->fg_gc,
-                            &item->entry->style->fg[GTK_STATE_SELECTED]);
-      gdk_gc_set_foreground(GTK_ITEM_ENTRY(item->entry)->bg_gc,
-                            &item->entry->style->bg[GTK_STATE_SELECTED]);
-      gtk_entry_set_text(GTK_ENTRY(item->entry), item->label);
-      gtk_entry_set_position(GTK_ENTRY(item->entry), 0);
-      gtk_widget_draw(item->entry, NULL);
-    }
-  } 
+      GtkStyle *style = gtk_style_copy(item->entry->style);
+      style->bg[GTK_STATE_ACTIVE] = style->base[GTK_STATE_SELECTED];
+      style->bg[GTK_STATE_NORMAL] = style->base[GTK_STATE_SELECTED];
+      style->text[GTK_STATE_ACTIVE] = style->text[GTK_STATE_SELECTED];
+      style->text[GTK_STATE_NORMAL] = style->text[GTK_STATE_SELECTED];
+      gtk_widget_set_style(item->entry, style);
+      gtk_style_unref(style);
 
+      switch(iconlist->mode){
+        case GTK_ICON_LIST_TEXT_RIGHT:
+          gtk_item_entry_set_text(GTK_ITEM_ENTRY(item->entry), 
+                                  item->label,
+                                  GTK_JUSTIFY_LEFT);
+          break;
+        case GTK_ICON_LIST_TEXT_BELOW:
+          gtk_item_entry_set_text(GTK_ITEM_ENTRY(item->entry), 
+                                  item->label,
+                                  GTK_JUSTIFY_CENTER);
+          break;
+        case GTK_ICON_LIST_ICON:
+        default:
+          break;
+      }
+    }
+  }
   if(item->state == GTK_STATE_SELECTED) return;
   iconlist->selection = g_list_append(iconlist->selection, item);
 
   item->state = GTK_STATE_SELECTED;  
   if(item->entry) gtk_widget_grab_focus(item->entry);
-
 }
 
 static void
@@ -745,14 +730,20 @@ unselect_icon(GtkIconList *iconlist, GtkIconListItem *item, GdkEvent *event)
 
   if(iconlist->mode != GTK_ICON_LIST_ICON){
    if(item->entry && GTK_WIDGET_REALIZED(item->entry)){
-     gdk_gc_set_foreground(GTK_ITEM_ENTRY(item->entry)->bg_gc, 
-                           &iconlist->background);
-     gdk_gc_set_foreground(GTK_ITEM_ENTRY(item->entry)->fg_gc,
-                           &item->entry->style->fg[GTK_STATE_NORMAL]);
+     GtkStyle *style = gtk_style_copy(item->entry->style);
+
+     style->bg[GTK_STATE_ACTIVE] = iconlist->background;
+     style->bg[GTK_STATE_NORMAL] = iconlist->background;
+     style->text[GTK_STATE_ACTIVE] = GTK_WIDGET(iconlist)->style->text[GTK_STATE_ACTIVE];
+     style->text[GTK_STATE_NORMAL] = GTK_WIDGET(iconlist)->style->text[GTK_STATE_NORMAL];
+     gtk_widget_set_style(item->entry, style);
+     gtk_style_unref(style);
+
      gtk_entry_select_region(GTK_ENTRY(item->entry), 0, 0);
      gtk_entry_set_text(GTK_ENTRY(item->entry), item->entry_label);
      gtk_entry_set_editable(GTK_ENTRY(item->entry), FALSE);
      gtk_widget_draw(item->entry, NULL);
+
    }
   }
 
@@ -769,7 +760,7 @@ unselect_all(GtkIconList *iconlist)
   while(selection){
     item = (GtkIconListItem *)selection->data;
     unselect_icon(iconlist, item, NULL);
-    selection = selection->next;
+    selection = iconlist->selection;
   }
 
   g_list_free(iconlist->selection);
@@ -810,7 +801,7 @@ gtk_icon_list_set_mode (GtkIconList *iconlist, GtkIconListMode mode)
     GtkIconListItem *item;
 
     item = (GtkIconListItem *)icons->data;
-    
+
     switch(mode){
       case GTK_ICON_LIST_TEXT_RIGHT:
         gtk_item_entry_set_justification(GTK_ITEM_ENTRY(item->entry), 
@@ -876,20 +867,6 @@ gtk_icon_list_destroy (GtkObject *object)
 }
 
 
-static void
-gtk_icon_list_finalize (GtkObject *object)
-{
-  GtkIconList *icon_list;
-
-  g_return_if_fail (object != NULL);
-  g_return_if_fail (GTK_IS_ICON_LIST (object));
-
-  icon_list = GTK_ICON_LIST (object);
-
-  if (GTK_OBJECT_CLASS (parent_class)->finalize)
-    (*GTK_OBJECT_CLASS (parent_class)->finalize) (object);
-}
-
 void
 gtk_icon_list_set_background (GtkIconList *iconlist, GdkColor *color)
 {
@@ -902,10 +879,12 @@ gtk_icon_list_set_background (GtkIconList *iconlist, GdkColor *color)
   widget = GTK_WIDGET(iconlist);
 
   iconlist->background = *color;
+
   style = gtk_style_copy(widget->style);
   style->bg[0] = iconlist->background;
+
   gtk_widget_set_style(widget, style);
-  gtk_style_set_background(widget->style, widget->window, GTK_STATE_NORMAL);
+  if(widget->window) gdk_window_set_background(widget->window, color);
   gtk_style_unref(style);
 
 }
@@ -916,18 +895,18 @@ entry_changed (GtkWidget *widget, gpointer data)
   GtkIconList *iconlist;
   GtkIconListItem *item;
   gboolean veto = TRUE;
-  gchar *text;
+  const gchar *text;
 
   iconlist = GTK_ICON_LIST(data);
   item = get_icon_from_entry(iconlist, widget);
   text = gtk_entry_get_text(GTK_ENTRY(widget));
 
-  gtk_signal_emit(GTK_OBJECT(data), signals[TEXT_CHANGED],
+  _gtkextra_signal_emit(GTK_OBJECT(data), signals[TEXT_CHANGED],
                   item, text, &veto);
 
   if(!veto) return veto; 
 
-  if(item->entry && GTK_EDITABLE(item->entry)->editable){
+  if(item->entry && gtk_editable_get_editable(GTK_EDITABLE(item->entry))){
     if(item->label) g_free(item->label);
     if(text) item->label = g_strdup(text); 
     if(item->entry_label) g_free(item->entry_label);
@@ -951,27 +930,29 @@ entry_in (GtkWidget *widget, GdkEventButton *event, gpointer data)
   if(iconlist->active_icon && iconlist->active_icon->entry == widget) 
           return FALSE;
 
-  gtk_signal_emit(GTK_OBJECT(iconlist), signals[ACTIVATE_ICON], &item, &veto);
+  _gtkextra_signal_emit(GTK_OBJECT(iconlist), signals[ACTIVATE_ICON], &item, &veto);
 
   if(!veto) return FALSE; 
   if(!deactivate_entry(iconlist)) return FALSE;
 
   if(item->state == GTK_STATE_SELECTED){
-   if(iconlist->is_editable && !GTK_EDITABLE(widget)->editable){
+   if(iconlist->is_editable && !gtk_editable_get_editable(GTK_EDITABLE(widget))){
      unselect_all(iconlist);
+
      gtk_entry_set_editable(GTK_ENTRY(widget), TRUE);
+     gtk_item_entry_set_cursor_visible(GTK_ITEM_ENTRY(widget), TRUE);
      if(item->label) gtk_entry_set_text(GTK_ENTRY(widget), item->label);
-     gtk_widget_draw(widget, NULL);
      iconlist->active_icon = item;
      item->state = GTK_STATE_NORMAL;
+
      if(GTK_WIDGET_DRAWABLE(widget)) 
        gdk_draw_rectangle(GTK_WIDGET(iconlist)->window,
-                          widget->style->fg_gc[GTK_STATE_NORMAL],
+                          widget->style->black_gc,
                           FALSE,
-                          iconlist->active_icon->entry->allocation.x+2,
-                          iconlist->active_icon->entry->allocation.y+2,
-                          iconlist->active_icon->entry->allocation.width-4,
-                          iconlist->active_icon->entry->allocation.height-4);
+                          iconlist->active_icon->entry->allocation.x-2,
+                          iconlist->active_icon->entry->allocation.y-2,
+                          iconlist->active_icon->entry->allocation.width+4,
+                          iconlist->active_icon->entry->allocation.height+4);
    }else{
      gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "button_press_event"); 
      if(iconlist->selection_mode == GTK_SELECTION_SINGLE ||
@@ -1112,7 +1093,7 @@ gtk_icon_list_real_add (GtkIconList *iconlist,
     switch(iconlist->mode){
       case GTK_ICON_LIST_TEXT_RIGHT:
         y += vspace;
-        if(y >= height-DEFAULT_COL_SPACING){
+        if(y >= height){
                 x += hspace;
                 y = iconlist->row_spacing;
         }
@@ -1121,12 +1102,12 @@ gtk_icon_list_real_add (GtkIconList *iconlist,
       case GTK_ICON_LIST_ICON:
       default:
         x += hspace;
-        if(x >= width-DEFAULT_COL_SPACING){
+        if(x >= width){
                 x = iconlist->col_spacing;
-                y += vspace ;
+                y += vspace;
         }
         break;
-  }
+    }
   } else {
     y = iconlist->row_spacing;
     x = iconlist->col_spacing;
@@ -1149,11 +1130,14 @@ gtk_icon_list_put (GtkIconList *iconlist,
   GtkWidget *widget;
   GtkRequisition req, req1, req2;
   gint text_width;
-  gint width, height;
+  gint width, height, old_width, old_height;
   GtkAllocation alloc;
 
   widget = GTK_WIDGET(iconlist);
 
+  old_width = width = GTK_WIDGET(iconlist)->allocation.width;
+  old_height = height = GTK_WIDGET(iconlist)->allocation.height;
+
   active_icon = iconlist->active_icon;
   gtk_icon_list_set_active_icon(iconlist, NULL);
 
@@ -1180,14 +1164,13 @@ gtk_icon_list_put (GtkIconList *iconlist,
      req1.width = MAX(req1.width, req.width);
   }
 
-
   if(iconlist->mode == GTK_ICON_LIST_ICON) 
           req2.width = req2.height = 0;
   else
           set_labels(iconlist, icon, label);
 
   text_width = 0;
-  if(label) text_width = gdk_string_width(icon->entry->style->font, label);
+  if(label) text_width = STRING_WIDTH(icon->entry, icon->entry->style->font_desc, label);
 
   gtk_fixed_put(GTK_FIXED(iconlist), icon->pixmap, 
                  x + req1.width/2 - icon->pixmap->requisition.width/2, 
@@ -1199,11 +1182,8 @@ gtk_icon_list_put (GtkIconList *iconlist,
   alloc.height =  req1.height;
   gtk_widget_size_allocate(icon->pixmap, &alloc);
 
-  width = GTK_WIDGET(iconlist)->allocation.width;
-  height = GTK_WIDGET(iconlist)->allocation.height;
-
   switch(iconlist->mode){
-      case GTK_ICON_LIST_TEXT_BELOW:
+   case GTK_ICON_LIST_TEXT_BELOW:
         gtk_item_entry_set_text(GTK_ITEM_ENTRY(icon->entry), icon->entry_label, 
                                 GTK_JUSTIFY_CENTER);
   	gtk_fixed_put(GTK_FIXED(iconlist), icon->entry, 
@@ -1215,10 +1195,10 @@ gtk_icon_list_put (GtkIconList *iconlist,
         alloc.height = req2.height;
         gtk_widget_size_allocate(icon->entry, &alloc);
 
-        if(y + req1.height + iconlist->icon_border + req2.height > height-DEFAULT_COL_SPACING) 
+        if(y + req1.height + iconlist->icon_border + req2.height > height) 
            height += req1.height + iconlist->icon_border + req2.height;
         break;
-      case GTK_ICON_LIST_TEXT_RIGHT:
+   case GTK_ICON_LIST_TEXT_RIGHT:
         gtk_item_entry_set_text(GTK_ITEM_ENTRY(icon->entry), icon->entry_label, 
                                 GTK_JUSTIFY_LEFT);
   	gtk_fixed_put(GTK_FIXED(iconlist), icon->entry, 
@@ -1230,20 +1210,20 @@ gtk_icon_list_put (GtkIconList *iconlist,
         alloc.height = req2.height;
         gtk_widget_size_allocate(icon->entry, &alloc);
 
-        if(x + req1.width + iconlist->icon_border + text_width > width-DEFAULT_COL_SPACING) 
+        if(x + req1.width + iconlist->icon_border + text_width > width) 
             width += req1.width + iconlist->icon_border + text_width;
 	break;
-      case GTK_ICON_LIST_ICON:
-      default: 
-        break;
+   case GTK_ICON_LIST_ICON:
+   default: ;
   }
 
   if(GTK_WIDGET_REALIZED(iconlist))
     if(iconlist->mode != GTK_ICON_LIST_ICON){
-      gdk_gc_set_foreground(GTK_ITEM_ENTRY(icon->entry)->bg_gc, 
-                            &iconlist->background);
-      gdk_gc_set_foreground(GTK_ITEM_ENTRY(icon->entry)->fg_gc, 
-                            &widget->style->fg[GTK_STATE_NORMAL]);
+      GtkStyle *style = gtk_style_copy(icon->entry->style);
+      style->bg[GTK_STATE_ACTIVE] = iconlist->background;
+      style->bg[GTK_STATE_NORMAL] = iconlist->background;
+      gtk_widget_set_style(icon->entry, style);
+      gtk_style_unref(style);
       gtk_widget_show(icon->entry);
     }
 
@@ -1260,6 +1240,7 @@ gtk_icon_list_put (GtkIconList *iconlist,
                 reorder_icons(iconlist);
 
   gtk_entry_set_editable(GTK_ENTRY(icon->entry), FALSE);
+
   gtk_signal_connect(GTK_OBJECT(icon->entry), "key_press_event",
                     (GtkSignalFunc)icon_key_press, iconlist);
   gtk_signal_connect(GTK_OBJECT(icon->entry), "button_press_event", 
@@ -1286,15 +1267,15 @@ set_labels(GtkIconList *iconlist, GtkIconListItem *icon, const gchar *label)
   entry_label[0] = label[0];
   entry_label[1] = '\0';
 
-  text_width = gdk_string_width(icon->entry->style->font, label);
-  point_width = gdk_string_width(icon->entry->style->font, "X");
+  text_width = STRING_WIDTH(icon->entry, icon->entry->style->font_desc, label);
+  point_width = STRING_WIDTH(icon->entry, icon->entry->style->font_desc, "X");
 
   max_width = iconlist->text_space;
 
   for(n = 0; n < strlen(label); n++){
      space = strlen(label) - n + 1;
      if(space > 3 && 
-        gdk_string_width(icon->entry->style->font, entry_label) + 
+        STRING_WIDTH(icon->entry, icon->entry->style->font_desc, entry_label) +
         3 * point_width > max_width) 
                                                    break;
      entry_label[n] = label[n];
@@ -1398,6 +1379,32 @@ gtk_icon_list_get_index(GtkIconList *iconlist, GtkIconListItem *item)
   return -1;
 }
 
+static void
+remove_from_fixed(GtkIconList *iconlist, GtkWidget *widget)
+{
+  GtkFixed *fixed;
+  GList *children;
+
+  fixed = GTK_FIXED(iconlist);
+  children = fixed->children;
+  while(children){
+    GtkFixedChild *child;
+
+    child = children->data;
+
+    if(child->widget == widget){
+      gtk_widget_unparent(widget);
+      fixed->children = g_list_remove_link (fixed->children, children);
+      g_list_free (children);
+      g_free (child);
+
+      break;
+    }
+
+    children = children->next;
+  }
+}
+
 static void
 pixmap_destroy(GtkPixmap* pixmap)
 {
@@ -1415,7 +1422,7 @@ pixmap_destroy(GtkPixmap* pixmap)
     if (bm) gdk_pixmap_unref(bm);
   }
 }
-
+  
 void
 gtk_icon_list_remove (GtkIconList *iconlist, GtkIconListItem *item)
 {
@@ -1433,11 +1440,16 @@ gtk_icon_list_remove (GtkIconList *iconlist, GtkIconListItem *item)
 
   if(icons){
      if(icon->state == GTK_STATE_SELECTED) unselect_icon(iconlist, icon, NULL);
+     if(icon == iconlist->active_icon) deactivate_entry(iconlist);
      pixmap_destroy(GTK_PIXMAP(icon->pixmap));
-     if(icon->entry && iconlist->mode != GTK_ICON_LIST_ICON)
-       gtk_container_remove(GTK_CONTAINER(iconlist), icon->entry);
-     if(icon->pixmap)
-       gtk_container_remove(GTK_CONTAINER(iconlist), icon->pixmap);
+     if(icon->entry && iconlist->mode != GTK_ICON_LIST_ICON){
+       remove_from_fixed(iconlist, icon->entry);
+       icon->entry = NULL;
+     }
+     if(icon->pixmap){
+       remove_from_fixed(iconlist, icon->pixmap);
+       icon->pixmap = NULL;
+     }
      if(icon->label){
         g_free(icon->label);
         icon->label = NULL;
@@ -1474,22 +1486,25 @@ gtk_icon_list_clear(GtkIconList *iconlist)
   GList *icons;
   GtkIconListItem *icon;
 
+  if(iconlist->num_icons == 0) return;
   if(!deactivate_entry(iconlist)) return;
+
   unselect_all(iconlist);
 
   icons = iconlist->icons;
 
-/*
-  gdk_threads_enter();
-*/
-
   while(icons){
     icon = (GtkIconListItem *) icons->data;
     pixmap_destroy(GTK_PIXMAP(icon->pixmap));
-    if(icon->entry && iconlist->mode != GTK_ICON_LIST_ICON)
-      gtk_container_remove(GTK_CONTAINER(iconlist), icon->entry);
-    if(icon->pixmap)
-      gtk_container_remove(GTK_CONTAINER(iconlist), icon->pixmap);
+    if(icon->entry && iconlist->mode != GTK_ICON_LIST_ICON){
+      remove_from_fixed(iconlist, icon->entry);
+      icon->entry = NULL;
+    }
+    if(icon->pixmap){
+      gtk_widget_hide(icon->pixmap);
+      remove_from_fixed(iconlist, icon->pixmap);
+      icon->pixmap = NULL;
+    }
     if(icon->label){
         g_free(icon->label);
         icon->label = NULL;
@@ -1500,17 +1515,16 @@ gtk_icon_list_clear(GtkIconList *iconlist)
     }
 
     g_free(icon);
+    icon = NULL;
 
     iconlist->icons = g_list_remove_link(iconlist->icons, icons);
     g_list_free_1(icons);
     icons = iconlist->icons;
   }
 
-/*
-  gdk_threads_leave();
-*/
-
   iconlist->icons = NULL;
+  iconlist->selection = NULL;
+  iconlist->active_icon = NULL;
   iconlist->num_icons = 0;
 }    
 
@@ -1701,3 +1715,4 @@ gtk_icon_list_get_icon_width       (GtkIconList *iconlist)
 {
   return(iconlist->icon_width);
 }
+
