@@ -50,6 +50,8 @@ GtkWidget *
 new_layer(GtkWidget *canvas)
 {
  gchar label[10];
+ GtkRequisition req;
+ gint size;
 
  nlayers++;
 
@@ -61,7 +63,9 @@ new_layer(GtkWidget *canvas)
  buttons[nlayers-1] = gtk_toggle_button_new_with_label(label);
 /* gtk_button_set_relief(GTK_BUTTON(buttons[nlayers-1]), GTK_RELIEF_NONE);
 */
- gtk_widget_set_usize(buttons[nlayers-1], 20, 20);
+ gtk_widget_size_request(buttons[nlayers-1], &req);
+ size = MAX(req.width,req.height);
+ gtk_widget_set_usize(buttons[nlayers-1], size, size);
  gtk_fixed_put(GTK_FIXED(canvas), buttons[nlayers-1], (nlayers-1)*20, 0);
  gtk_widget_show(buttons[nlayers-1]);
 
