@@ -5,7 +5,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <glib.h>
 #include <string.h>
-#include "gtkcombobox.h"
+#include "gtkcombobutton.h"
 #include "gtkbordercombo.h"
 #include "gtkcolorcombo.h"
 #include "gtktogglecombo.h"
@@ -123,9 +123,11 @@ build_menu(GtkWidget *sheet)
 
 	for (i=0; i < (sizeof(items)/sizeof(items[0])) ; i++){
 	 	item=gtk_menu_item_new_with_label(items[i]);
+
 	 	gtk_signal_connect(GTK_OBJECT(item),"activate",
 				   (GtkSignalFunc) popup_activated,
 				   items[i]);
+
                 GTK_WIDGET_SET_FLAGS (item, GTK_SENSITIVE | GTK_CAN_FOCUS);
                 switch(i){
                   case 2:
@@ -177,11 +179,11 @@ do_popup(GtkWidget *widget, GdkEventButton *event, gpointer data)
 		   event->button, event->time);
    }
 
-   return TRUE;
+   return FALSE;
 }
 
 void
-format_text (GtkSheet *sheet, gchar *text, gint *justification, char *label)
+format_text (GtkSheet *sheet, const gchar *text, gint *justification, char *label)
 {
   double auxval;
   int digspace=0;
@@ -193,10 +195,16 @@ format_text (GtkSheet *sheet, gchar *text, gint *justification, char *label)
   int nonzero=FALSE;
   int ipos;
   char nchar;
+  PangoContext *context;
+  PangoFontMetrics *metrics;
   enum {EMPTY, TEXT, NUMERIC};
 
+  context = gtk_widget_get_pango_context(GTK_WIDGET(sheet));
+  metrics = pango_context_get_metrics(context, GTK_WIDGET(sheet)->style->font_desc, pango_context_get_language(context));
+  char_width = pango_font_metrics_get_approximate_char_width(metrics);
+  pango_font_metrics_unref(metrics);
+
   cell_width=sheet->column[sheet->active_cell.col].width;
-  char_width = gdk_char_width (GTK_WIDGET(sheet)->style->font,(gchar)'X');
   space= (double)cell_width/(double)char_width;
 
   intspace=MIN(space, DEFAULT_SPACE);
@@ -283,7 +291,7 @@ parse_numbers(GtkWidget *widget, gpointer data)
 
 }
 
-void
+gint 
 clipboard_handler(GtkWidget *widget, GdkEventKey *key)
 {
   GtkSheet *sheet;
@@ -301,6 +309,8 @@ clipboard_handler(GtkWidget *widget, GdkEventKey *key)
     if(key->keyval=='x' || key->keyval == 'X')
             gtk_sheet_unclip_range(sheet);    
   }
+
+  return FALSE;
 }
 
 void 
@@ -329,9 +339,10 @@ move_handler(GtkWidget *widget, GtkSheetRange *old_range,
 
 }
 
-gint change_entry(GtkWidget *widget, 
-                    gint row, gint col, gint *new_row, gint *new_col,
-                    gpointer data)
+gboolean
+change_entry(GtkWidget *widget, 
+             gint row, gint col, gint *new_row, gint *new_col,
+             gpointer data)
 {
   GtkSheet *sheet;
 
@@ -359,7 +370,7 @@ void alarm_change(GtkWidget *widget, gint row, gint col,
  printf("CHANGE CELL: %d %d\n",row,col);
 }
 
-void alarm_activate(GtkWidget *widget, gint row, gint col, 
+gboolean alarm_activate(GtkWidget *widget, gint row, gint col, 
                     gpointer data)
 {
  GtkSheetRange range;
@@ -371,12 +382,15 @@ void alarm_activate(GtkWidget *widget, gint row, gint col,
 /*
  gtk_sheet_range_set_justification(GTK_SHEET(widget), &range, GTK_JUSTIFY_LEFT);
 */
+ return TRUE;
 }
 
-void alarm_deactivate(GtkWidget *widget, gint row, gint col, 
+gboolean alarm_deactivate(GtkWidget *widget, gint row, gint col, 
                       gpointer data)
 {
  GtkSheetRange range;
+ gchar *text;
+ GtkSheet *sheet = GTK_SHEET(widget);
 
  printf("DEACTIVATE CELL: %d %d\n",row,col);
 
@@ -385,9 +399,17 @@ void alarm_deactivate(GtkWidget *widget, gint row, gint col,
 /*
  gtk_sheet_range_set_justification(GTK_SHEET(widget), &range, GTK_JUSTIFY_RIGHT);
 */
+  text = g_strdup(gtk_sheet_cell_get_text(GTK_SHEET(widget), row, col));
+
+  if(text && strlen(text) > 0){
+    gtk_sheet_set_cell_text(sheet, row, col, text);
+    g_free(text);
+  }
+
+ return TRUE;
 }
 
-gint alarm_traverse(GtkWidget *widget, 
+gboolean alarm_traverse(GtkWidget *widget, 
                     gint row, gint col, gint *new_row, gint *new_col,
                     gpointer data)
 {
@@ -412,9 +434,9 @@ build_example1(GtkWidget *widget)
  GdkRectangle area;
  GdkColor color;
  GdkColormap *colormap;
- GdkFont *font;
- gchar font_name1[]="-bitstream-charter-bold-r-normal--36-360-72-72-p-0-iso8859-1";
- gchar font_name2[]="-bitstream-charter-bold-r-normal--28-280-72-72-p-0-iso8859-1";
+ PangoFontDescription *font;
+ gchar font_name1[]="Arial 36";
+ gchar font_name2[]="Arial 28";
 
  gchar name[10];
  gint i;
@@ -432,18 +454,17 @@ build_example1(GtkWidget *widget)
  for(i=0; i<=sheet->maxcol; i++){
    name[0]='A'+i;
    name[1]='\0';
+
    gtk_sheet_column_button_add_label(sheet, i, name);
    gtk_sheet_set_column_title(sheet, i, name);
  }
 
  gtk_sheet_row_button_add_label(sheet, 0, "This is\na multiline\nlabel");
  gtk_sheet_row_button_add_label(sheet, 1, "This is long label");
-
 /*
  gtk_sheet_column_button_add_label(sheet, 7, "This is\na multiline\nlabel");
  gtk_sheet_column_button_add_label(sheet, 8, "This is long label");
-*/
-
+*/ 
  gtk_sheet_row_button_justify(sheet, 0, GTK_JUSTIFY_RIGHT);
 
  range.row0=1;
@@ -453,11 +474,8 @@ build_example1(GtkWidget *widget)
 
  gtk_sheet_clip_range(sheet, &range);
 
- font=gdk_font_load(font_name2);
- if(!font)
-   g_warning ("Couldn't load %s", font_name2);
- else
-   gtk_sheet_range_set_font(sheet, &range, font);
+ font = pango_font_description_from_string (font_name2);
+ gtk_sheet_range_set_font(sheet, &range, font);
 
  gdk_color_parse("red", &color);
  gdk_color_alloc(gtk_widget_get_colormap(widget), &color);
@@ -468,11 +486,8 @@ build_example1(GtkWidget *widget)
 
  range.row0=2;
 
- font=gdk_font_load(font_name1);
- if(!font)
-   g_warning ("Couldn't load %s", font_name1);
- else
-   gtk_sheet_range_set_font(sheet, &range, font);
+ font = pango_font_description_from_string (font_name1);
+ gtk_sheet_range_set_font(sheet, &range, font);
 
  gdk_color_parse("blue", &color);
  gdk_color_alloc(gtk_widget_get_colormap(widget), &color);
@@ -509,7 +524,6 @@ build_example1(GtkWidget *widget)
  gtk_sheet_set_cell(sheet, 10, 1, GTK_JUSTIFY_LEFT,
  "You can add buttons, charts, pixmaps, and other widgets");
 
-
  gtk_signal_connect(GTK_OBJECT(sheet),
                     "key_press_event",
                     (GtkSignalFunc) clipboard_handler, 
@@ -572,6 +586,7 @@ build_example1(GtkWidget *widget)
  gdk_pixmap_unref(pixmap);
  gdk_bitmap_unref(mask);
 
+
  pixmap=gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask, NULL,
  				   smile_xpm);
  smile = gtk_pixmap_new(pixmap, mask);
@@ -599,6 +614,7 @@ build_example1(GtkWidget *widget)
                     NULL);
 */
 
+
 }
 
 
@@ -611,13 +627,11 @@ build_example2(GtkWidget *widget)
  GtkWidget *b;
  
  sheet=GTK_SHEET(widget);
- gtk_sheet_set_autoscroll(sheet, FALSE);
+ gtk_sheet_set_autoscroll(sheet, TRUE);
 /*
- gtk_sheet_set_justify_entry(sheet, FALSE);
+ gtk_sheet_set_justify_entry(sheet, TRUE);
 */
-/*
  gtk_sheet_set_selection_mode(sheet, GTK_SELECTION_SINGLE);
-*/
 
  range.row0=0;
  range.rowi=2;
@@ -644,7 +658,6 @@ build_example2(GtkWidget *widget)
  gtk_sheet_row_set_sensitivity(sheet, 1, FALSE);
  gtk_sheet_row_set_sensitivity(sheet, 2, FALSE);
 */
-
  gtk_sheet_set_cell(sheet, 0,2, GTK_JUSTIFY_CENTER,
                     "Click the right mouse button to display a popup");
  gtk_sheet_set_cell(sheet, 1,2, GTK_JUSTIFY_CENTER,
@@ -652,9 +665,11 @@ build_example2(GtkWidget *widget)
  gtk_sheet_set_cell(sheet, 2,2, GTK_JUSTIFY_CENTER,
                     "(Try typing numbers)");
  gtk_sheet_set_active_cell(sheet, 3, 0);
+
 /*
  gtk_sheet_set_update_policy(sheet, GTK_UPDATE_CONTINUOUS, GTK_UPDATE_CONTINUOUS);
 */
+
  gtk_signal_connect(GTK_OBJECT(sheet),
                     "button_press_event",
                     (GtkSignalFunc) do_popup, 
@@ -665,16 +680,29 @@ build_example2(GtkWidget *widget)
 		    (GtkSignalFunc) parse_numbers,
                     NULL);                   
 
+ gtk_signal_connect(GTK_OBJECT(sheet),
+                    "activate",
+                    (GtkSignalFunc) alarm_activate, 
+                    NULL);
+
+ gtk_signal_connect(GTK_OBJECT(sheet),
+                    "deactivate",
+                    (GtkSignalFunc) alarm_deactivate, 
+                    NULL);
+
+ gtk_signal_connect(GTK_OBJECT(sheet),
+                    "traverse",
+                    (GtkSignalFunc) alarm_traverse, 
+                    NULL);
+
  gtk_sheet_set_row_height(GTK_SHEET(sheets[1]), 12, 60);
  b = gtk_button_new_with_label("GTK_FILL");
  gtk_sheet_attach(GTK_SHEET(sheets[1]), b, 12, 2, GTK_FILL, GTK_FILL, 5, 5);
  gtk_widget_show(b);
  b = gtk_button_new_with_label("GTK_EXPAND");
- gtk_sheet_attach(GTK_SHEET(sheets[1]), b, 12, 3, GTK_EXPAND, GTK_EXPAND, 5, 5);
- gtk_widget_show(b);
+ gtk_sheet_attach(GTK_SHEET(sheets[1]), b, 12, 3, GTK_EXPAND, GTK_EXPAND, 5, 5); gtk_widget_show(b);
  b = gtk_button_new_with_label("GTK_SHRINK");
- gtk_sheet_attach(GTK_SHEET(sheets[1]), b, 12, 4, GTK_SHRINK, GTK_SHRINK, 5, 5);
- gtk_widget_show(b);
+ gtk_sheet_attach(GTK_SHEET(sheets[1]), b, 12, 4, GTK_SHRINK, GTK_SHRINK, 5, 5); gtk_widget_show(b);
 }
 
 void
@@ -732,12 +760,12 @@ build_example3(GtkWidget *widget)
 
  gtk_sheet_change_entry(sheet, gtk_combo_get_type());
 
+
  gtk_signal_connect(GTK_OBJECT(sheet),
                     "traverse",
                     (GtkSignalFunc) change_entry, 
                     NULL);
 
-
 }
 
 /*
@@ -758,7 +786,7 @@ void
 set_cell(GtkWidget *widget, gchar *insert, gint text_legth, gint position, 
          gpointer data)
 {
-  char *text;
+  const char *text;
   GtkEntry *sheet_entry;
 
   sheet_entry = GTK_ENTRY(gtk_sheet_get_entry(GTK_SHEET(widget)));
@@ -773,7 +801,7 @@ set_cell(GtkWidget *widget, gchar *insert, gint text_legth, gint position,
 void
 show_sheet_entry(GtkWidget *widget, gpointer data)
 {
- char *text;
+ const char *text;
  GtkSheet *sheet;
  GtkEntry *sheet_entry;
  gint cur_page;
@@ -815,7 +843,7 @@ activate_sheet_entry(GtkWidget *widget, gpointer data)
 void 
 show_entry(GtkWidget *widget, gpointer data)
 {
- char *text; 
+ const char *text; 
  GtkSheet *sheet;
  GtkWidget * sheet_entry;
  gint cur_page;
@@ -897,7 +925,7 @@ activate_sheet_cell(GtkWidget *widget, gint row, gint column, gpointer data)
   GtkSheet *sheet;
   GtkEntry *sheet_entry;
   char cell[100];
-  char *text;
+  const char *text;
   GtkSheetCellAttr attributes;
 
   sheet=GTK_SHEET(widget);
@@ -1080,7 +1108,7 @@ change_fg(GtkWidget *widget, gint i, GdkColor *color)
    tmp_gc=gdk_gc_new(window->window);
    gdk_gc_set_foreground(tmp_gc,color);
    gdk_draw_rectangle(GTK_PIXMAP(fg_pixmap)->pixmap,
-                      tmp_gc, TRUE, 5,20,16,4);
+		      tmp_gc, TRUE, 5,20,16,4);
    gtk_widget_queue_draw(fg_pixmap); 
    gdk_gc_unref(tmp_gc);
 }
@@ -1100,8 +1128,8 @@ change_bg(GtkWidget *widget, gint i, GdkColor *color)
    tmp_gc=gdk_gc_new(window->window);
    gdk_gc_set_foreground(tmp_gc,color);
    gdk_draw_rectangle(GTK_PIXMAP(bg_pixmap)->pixmap,
-                      tmp_gc, TRUE, 4,20,18,4);
-   gtk_widget_draw(bg_pixmap, NULL);
+		      tmp_gc, TRUE, 4,20,18,4);
+   gtk_widget_draw(bg_pixmap, NULL); 
    gdk_gc_unref(tmp_gc);
 }
 
@@ -1164,10 +1192,9 @@ new_font(GtkFontCombo *font_combo, gpointer data)
   cur_page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
   current=GTK_SHEET(sheets[cur_page]);
 
-  gtk_sheet_range_set_font(current, &current->range, font_combo->font);
+  gtk_sheet_range_set_font(current, &current->range, gtk_font_combo_get_font_description(font_combo));
 }
 
-
 int main(int argc, char *argv[]) 
 {
   GtkWidget *label; 
@@ -1234,11 +1261,10 @@ int main(int argc, char *argv[])
   gtk_signal_connect(GTK_OBJECT(show_column_titles), "clicked",
 		     (GtkSignalFunc) do_show_column_titles, NULL);
 
-
   gtk_box_pack_start(GTK_BOX(main_vbox), show_hide_box, FALSE, TRUE, 0);
   gtk_widget_show(show_hide_box);
 
-  toolbar=gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_ICONS);
+  toolbar=gtk_toolbar_new();
 
   gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 
@@ -1257,6 +1283,7 @@ int main(int argc, char *argv[])
   gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
 			    left_button, "justify left", "justify left");
   gtk_widget_show(left_button);
+
   gtk_signal_connect(GTK_OBJECT(left_button),"released",
 		     (GtkSignalFunc) justify_left, NULL);
 
@@ -1264,13 +1291,16 @@ int main(int argc, char *argv[])
   gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
 			    center_button, "justify center", "justify center");
   gtk_widget_show(center_button);
+
   gtk_signal_connect(GTK_OBJECT(center_button), "released",
 		     (GtkSignalFunc) justify_center, NULL);
 
+
   right_button = gtk_toggle_button_new();
   gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
 			    right_button, "justify right", "justify right");
   gtk_widget_show(right_button);
+
   gtk_signal_connect(GTK_OBJECT(right_button), "released",
 		     (GtkSignalFunc) justify_right, NULL);
 
@@ -1279,7 +1309,7 @@ int main(int argc, char *argv[])
   bordercombo=gtk_border_combo_new();
   gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
 			    bordercombo, "border", "border");
-  gtk_widget_set_usize(GTK_COMBO_BOX(bordercombo)->button, 32, 32);
+  gtk_widget_set_usize(GTK_COMBO_BUTTON(bordercombo)->button, 32, 32);
   gtk_widget_show(bordercombo);
 
   gtk_signal_connect(GTK_OBJECT(bordercombo),
@@ -1291,6 +1321,7 @@ int main(int argc, char *argv[])
   gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
 			    fgcolorcombo, "font color", "font color");
   gtk_widget_show(fgcolorcombo);
+
   gtk_signal_connect(GTK_OBJECT(fgcolorcombo),
 		     "changed", (GtkSignalFunc)change_fg, NULL);
 
@@ -1300,6 +1331,7 @@ int main(int argc, char *argv[])
   gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
 			    bgcolorcombo, "background color", "background color");
   gtk_widget_show(bgcolorcombo);
+
   gtk_signal_connect(GTK_OBJECT(bgcolorcombo),
 		     "changed", (GtkSignalFunc)change_bg, NULL);
 
@@ -1308,7 +1340,7 @@ int main(int argc, char *argv[])
   toggle_combo = gtk_toggle_combo_new(5, 5);
   gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar),
 			    toggle_combo, "test", "test");
-  gtk_widget_set_usize(GTK_COMBO_BOX(toggle_combo)->button, 32, 32);
+  gtk_widget_set_usize(GTK_COMBO_BUTTON(toggle_combo)->button, 32, 32);
   gtk_widget_show(toggle_combo);
 
   gtk_box_pack_start(GTK_BOX(main_vbox), toolbar, FALSE, TRUE, 0);
@@ -1351,16 +1383,15 @@ int main(int argc, char *argv[])
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), GTK_WIDGET(scrolled_windows[i]),
    label);
 
-
    gtk_signal_connect(GTK_OBJECT(gtk_sheet_get_entry(GTK_SHEET(sheets[i]))),
 		      "changed", (GtkSignalFunc)show_entry, NULL);
 
    gtk_signal_connect(GTK_OBJECT(sheets[i]),
 		      "activate", (GtkSignalFunc)activate_sheet_cell,
 		      NULL);
-
   }
 
+
   gtk_signal_connect(GTK_OBJECT(entry),
 		      "changed", (GtkSignalFunc)show_sheet_entry, NULL);
 
@@ -1368,6 +1399,7 @@ int main(int argc, char *argv[])
 		      "activate", (GtkSignalFunc)activate_sheet_entry,
 		      NULL);
 
+
   build_example1(sheets[0]); 
   build_example2(sheets[1]);
   build_example3(sheets[2]);
@@ -1404,7 +1436,7 @@ int main(int argc, char *argv[])
 				    paint);
 
   bg_pixmap=gtk_pixmap_new(pixmap, mask);
-  gtk_container_add(GTK_CONTAINER(GTK_COMBO_BOX(bgcolorcombo)->button),
+  gtk_container_add(GTK_CONTAINER(GTK_COMBO_BUTTON(bgcolorcombo)->button),
 		    bg_pixmap);
 
   gtk_widget_show(bg_pixmap);
@@ -1416,7 +1448,7 @@ int main(int argc, char *argv[])
 
 
   fg_pixmap=gtk_pixmap_new(pixmap, mask);
-  gtk_container_add(GTK_CONTAINER(GTK_COMBO_BOX(fgcolorcombo)->button),
+  gtk_container_add(GTK_CONTAINER(GTK_COMBO_BUTTON(fgcolorcombo)->button),
 		    fg_pixmap);
 
   gtk_widget_show(fg_pixmap);
